package netscape.ldap;

public class LDAPSearchConstraints implements Cloneable {
   private int timeLimit;
   private int deref;
   private int maxRes;
   private boolean referrals;
   private int batch;
   private LDAPRebind m_rebind_proc;
   private int m_hop_limit;
   private LDAPControl[] m_clientControls;
   private LDAPControl[] m_serverControls;
   private transient int m_maxBacklog = 100;

   public LDAPSearchConstraints() {
      this.timeLimit = 0;
      this.deref = 0;
      this.maxRes = 1000;
      this.referrals = false;
      this.batch = 1;
      this.m_rebind_proc = null;
      this.m_hop_limit = 10;
      this.m_clientControls = null;
      this.m_serverControls = null;
   }

   public LDAPSearchConstraints(int var1, int var2, int var3, boolean var4, int var5, LDAPRebind var6, int var7) {
      this.timeLimit = var1;
      this.deref = var2;
      this.maxRes = var3;
      this.referrals = var4;
      this.batch = var5;
      this.m_rebind_proc = var6;
      this.m_hop_limit = var7;
      this.m_clientControls = null;
      this.m_serverControls = null;
   }

   public int getTimeLimit() {
      return this.timeLimit * 1000;
   }

   public int getDereference() {
      return this.deref;
   }

   public int getMaxResults() {
      return this.maxRes;
   }

   public boolean getReferrals() {
      return this.referrals;
   }

   public int getBatchSize() {
      return this.batch;
   }

   public LDAPRebind getRebindProc() {
      return this.m_rebind_proc;
   }

   public int getHopLimit() {
      return this.m_hop_limit;
   }

   public LDAPControl[] getClientControls() {
      return this.m_clientControls;
   }

   public LDAPControl[] getServerControls() {
      return this.m_serverControls;
   }

   public void setTimeLimit(int var1) {
      if (var1 != 0) {
         this.timeLimit = Math.max(1, (var1 + 500) / 1000);
      }

   }

   public void setDereference(int var1) {
      this.deref = var1;
   }

   public void setMaxResults(int var1) {
      this.maxRes = var1;
   }

   public void setReferrals(boolean var1) {
      this.referrals = var1;
   }

   public void setBatchSize(int var1) {
      this.batch = var1;
   }

   public void setRebindProc(LDAPRebind var1) {
      this.m_rebind_proc = var1;
   }

   public void setHopLimit(int var1) {
      this.m_hop_limit = var1;
   }

   public void setClientControls(LDAPControl var1) {
      this.m_clientControls = new LDAPControl[1];
      this.m_clientControls[0] = var1;
   }

   public void setClientControls(LDAPControl[] var1) {
      this.m_clientControls = var1;
   }

   public void setServerControls(LDAPControl var1) {
      this.m_serverControls = new LDAPControl[1];
      this.m_serverControls[0] = var1;
   }

   public void setServerControls(LDAPControl[] var1) {
      this.m_serverControls = var1;
   }

   public void setMaxBacklog(int var1) {
      this.m_maxBacklog = var1;
   }

   public int getMaxBacklog() {
      return this.m_maxBacklog;
   }

   public Object clone() {
      LDAPSearchConstraints var1 = new LDAPSearchConstraints();
      var1.timeLimit = this.timeLimit;
      var1.deref = this.deref;
      var1.maxRes = this.maxRes;
      var1.referrals = this.referrals;
      var1.batch = this.batch;
      var1.m_rebind_proc = this.m_rebind_proc;
      var1.m_hop_limit = this.m_hop_limit;
      if (this.m_clientControls != null && this.m_clientControls.length > 0) {
         var1.m_clientControls = new LDAPControl[this.m_clientControls.length];

         for(int var2 = 0; var2 < this.m_clientControls.length; ++var2) {
            var1.m_clientControls[var2] = this.m_clientControls[var2];
         }
      }

      if (this.m_serverControls != null && this.m_serverControls.length > 0) {
         var1.m_serverControls = new LDAPControl[this.m_serverControls.length];

         for(int var3 = 0; var3 < this.m_serverControls.length; ++var3) {
            var1.m_serverControls[var3] = this.m_serverControls[var3];
         }
      }

      return var1;
   }
}
