package netscape.ldap.client.opers;

import netscape.ldap.ber.stream.BERBoolean;
import netscape.ldap.ber.stream.BERConstruct;
import netscape.ldap.ber.stream.BERElement;
import netscape.ldap.ber.stream.BEREnumerated;
import netscape.ldap.ber.stream.BERInteger;
import netscape.ldap.ber.stream.BEROctetString;
import netscape.ldap.ber.stream.BERSequence;
import netscape.ldap.ber.stream.BERTag;
import netscape.ldap.client.JDAPFilter;

public class JDAPSearchRequest extends JDAPBaseDNRequest implements JDAPProtocolOp {
   public static final int BASE_OBJECT = 0;
   public static final int SINGLE_LEVEL = 1;
   public static final int WHOLE_SUBTREE = 2;
   public static final int NEVER_DEREF_ALIASES = 0;
   public static final int DEREF_IN_SEARCHING = 1;
   public static final int DEREF_FINDING_BASE_OBJ = 2;
   public static final int DEREF_ALWAYS = 3;
   protected String m_base_dn;
   protected int m_scope;
   protected int m_deref;
   protected int m_size_limit;
   protected int m_time_limit;
   protected boolean m_attrs_only;
   protected String m_filter;
   protected String[] m_attrs;

   public JDAPSearchRequest(String var1, int var2, int var3, int var4, int var5, boolean var6, String var7, String[] var8) {
      this.m_base_dn = var1;
      this.m_scope = var2;
      this.m_deref = var3;
      this.m_size_limit = var4;
      this.m_time_limit = var5;
      this.m_attrs_only = var6;
      this.m_filter = var7;
      this.m_attrs = var8;
   }

   public int getType() {
      return 3;
   }

   public void setBaseDN(String var1) {
      this.m_base_dn = var1;
   }

   public String getBaseDN() {
      return this.m_base_dn;
   }

   public BERElement getBERElement() {
      BERSequence var1 = new BERSequence();
      ((BERConstruct)var1).addElement(new BEROctetString(this.m_base_dn));
      ((BERConstruct)var1).addElement(new BEREnumerated(this.m_scope));
      ((BERConstruct)var1).addElement(new BEREnumerated(this.m_deref));
      ((BERConstruct)var1).addElement(new BERInteger(this.m_size_limit));
      ((BERConstruct)var1).addElement(new BERInteger(this.m_time_limit));
      ((BERConstruct)var1).addElement(new BERBoolean(this.m_attrs_only));
      JDAPFilter var2 = JDAPFilter.getFilter(this.m_filter);
      if (var2 == null) {
         return null;
      } else {
         ((BERConstruct)var1).addElement(var2.getBERElement());
         BERSequence var3 = new BERSequence();
         if (this.m_attrs != null) {
            for(int var4 = 0; var4 < this.m_attrs.length; ++var4) {
               ((BERConstruct)var3).addElement(new BEROctetString(this.m_attrs[var4]));
            }
         }

         ((BERConstruct)var1).addElement(var3);
         BERTag var5 = new BERTag(99, var1, true);
         return var5;
      }
   }

   public String toString() {
      String var1 = null;
      if (this.m_attrs != null) {
         for(int var2 = 0; var2 < this.m_attrs.length; ++var2) {
            if (var2 != 0) {
               var1 = var1 + "+";
            }

            var1 = var1 + this.m_attrs[var2];
         }
      }

      return "JDAPSearchRequest {baseObject=" + this.m_base_dn + ", scope=" + this.m_scope + ", derefAliases=" + this.m_deref + ",sizeLimit=" + this.m_size_limit + ", timeLimit=" + this.m_time_limit + ", attrsOnly=" + this.m_attrs_only + ", filter=" + this.m_filter + ", attributes=" + var1 + "}";
   }
}
